import os
import tarfile
import zipfile
import io

import fleep


def extract_as_file(path, fname, fileobj):
    with open(os.path.join(path, fname), 'wb') as f:
        f.write(fileobj.read())


# https://stackoverflow.com/questions/10060069/safely-extract-zip-or-tar-using-python
resolved = lambda x: os.path.realpath(os.path.abspath(x))


def badpath(path, base):
    # joinpath will ignore base if path is absolute
    return not resolved(os.path.join(base, path)).startswith(base)


def badlink(info, base):
    # Links are interpreted relative to the directory containing the link
    tip = resolved(os.path.join(base, os.path.dirname(info.name)))
    return badpath(info.linkname, base=tip)


def safemembers(members):
    base = resolved(".")

    for finfo in members:
        if badpath(finfo.name, base):
            print(f"{finfo.name} is blocked (illegal path)")
        elif finfo.issym() and badlink(finfo, base):
            print(f"{finfo.name} is blocked: Hard link to {finfo.linkname}")
        elif finfo.islnk() and badlink(finfo, base):
            print(f"{finfo.name} is blocked: Symlink to {finfo.linkname}")
        else:
            yield finfo


def auto_extract_tar_gz(path, fileobj):
    with tarfile.open(fileobj=fileobj, mode='r:gz') as t:
        t.extractall(path=path, members=safemembers(t))
        return True


def auto_extract_zip(path, fileobj):
    with zipfile.ZipFile(fileobj) as z:
        for m in z.infolist():
            z.extract(m, path=path)


def auto_extract(path, fname, fileobj):
    try:
        if fname.endswith('.tar.gz') or fname.endswith('.tgz'):
            auto_extract_tar_gz(path, fileobj)
        elif fname.endswith('.zip'):
            auto_extract_zip(path, fileobj)
        else:
            raise ValueError(f'Unknown file extension for file {fname}')
    except tarfile.ReadError as t:
        raise ValueError('Failed to extract tarfile') from t
    except zipfile.BadZipfile as z:
        raise ValueError('Failed to extract zipfile') from z


def extract_all_submissions(path, sub, by_group=False):
    with zipfile.ZipFile(io.BytesIO(sub)) as z:
        if not by_group:
            print(f'Extracting {len(z.namelist())} submissions to {path}')
            # TODO Not fully sanitized.
            for n in z.namelist():
                student = n.split('_')[0]
                userpath = os.path.join(path, student)
                fname = os.path.basename(n)
                os.makedirs(userpath, exist_ok=True)
                with z.open(n) as s:

                    try:
                        auto_extract(userpath, fname, s)
                    except ValueError:
                        print(f'Auto-Extraction for {student} failed. Attempting fallback')

                        try:
                            s.seek(0)
                            magic_bytes = s.read(128)
                            s.seek(0)
                            info = fleep.get(magic_bytes)
                        except io.UnsupportedOperation:
                            # Workaround for Python < 3.7
                            with z.open(n) as s2:
                                magic_bytes = s2.read(128)
                                info = fleep.get(magic_bytes)

                        ext = ''
                        if 'zip' in info.extension:
                            ext += '.zip'
                        elif 'gz' in info.extension:
                            ext += '.tar.gz'  # TODO What if its just gz?
                        if ext != '':
                            fname += ext
                            print(f'Magic bytes indicate file type: {ext[1:]}')

                        try:
                            auto_extract(userpath, fname, s)
                        except ValueError:
                            print('Fallback failed. Writing submission as is.')
                            extract_as_file(userpath, fname, s)

        else:

            groups = set(n.split('-')[0] for n in z.namelist())
            print(f'Extracting {len(groups)} group submissions to {path}')
            for g in groups:
                group_path = os.path.join(path, g)
                os.makedirs(group_path, exist_ok=True)

                # We assume all submissions by the same group are identical
                group_subs = list(filter(lambda s: s.startswith(g + '-'),
                                         z.namelist()))
                fname = os.path.basename(group_subs[0])
                with z.open(group_subs[0]) as s:
                    try:
                        auto_extract(group_path, fname, s)
                    except ValueError as e:
                        print(f'Auto-Extraction for submission from group {g}! Attempting fallback.')

                        try:
                            s.seek(0)
                            magic_bytes = s.read(128)
                            s.seek(0)
                            info = fleep.get(magic_bytes)
                        except io.UnsupportedOperation:
                            # Workaround for Python < 3.7
                            with z.open(group_subs[0]) as s2:
                                magic_bytes = s2.read(128)
                                info = fleep.get(magic_bytes)

                        ext = ''
                        if 'zip' in info.extension:
                            ext += '.zip'
                        elif 'gz' in info.extension:
                            ext += '.tar.gz'  # TODO What if its just gz?

                        if ext != '':
                            fname += ext
                            print(f'Magic bytes indicate file type: {ext[1:]}')

                        try:
                            auto_extract(group_path, fname, s)
                        except ValueError:
                            print('Fallback failed. Writing submission as is.')
                            extract_as_file(group_path, fname, s)



import requests
import getpass
import dataclasses
import bs4
import sys
import argparse

MOSES_URL = 'https://moseskonto.tu-berlin.de'
MOSES_SHIB_URL = MOSES_URL + '/moses/shibboleth/login'
MOSES_SEARCH_URL = MOSES_URL + '/moses/administration/personen/suche.html'

@dataclasses.dataclass
class Person:
    name: str
    surname: str
    username: str
    role: str
    id: int
    group: str


def login(user, pw):
    s = requests.Session()
    s.get(MOSES_URL)           # Create a flow for this session

    r = s.get(MOSES_SHIB_URL)  # We will be redirected so save response

    # I have no idea what this does but without this first post we cant login
    payload = {
        'shib_idp_ls_exception.shib_idp_session_ss':'',
        'shib_idp_ls_success.shib_idp_session_ss': True,
        'shib_idp_ls_value.shib_idp_session_ss': '',
        'shib_idp_ls_exception.shib_idp_persistent_ss': '',
        'shib_idp_ls_success.shib_idp_persistent_ss': True,
        'shib_idp_ls_value.shib_idp_persistent_ss': '',
        'shib_idp_ls_supported': '',
        '_eventId_proceed': True
    }
    r = s.post(r.url, data=payload)

    payload = {
        'j_username': user,
        'j_password': pw,
        '_eventId_proceed': ''
    }

    r = s.post(r.url, data=payload)

    for h in r.history:
        for c in h.cookies.keys():
            if '_shibsession_' in c:
                return s

    return None


def search_people(session: requests.Session,
                  lastname='',
                  surname='',
                  email='',
                  username='',
                  student_id=''):
    r = session.get(MOSES_SEARCH_URL)
    soup = bs4.BeautifulSoup(r.text, 'html.parser')

    view_state = soup.find('input', attrs={'name': 'javax.faces.ViewState'})['value']
    jfwid = soup.find('input', attrs={'name': 'javax.faces.ClientWindow'})['value']

    payload = {
        "javax.faces.source": "j_idt100:j_idt117",
        'javax.faces.partial.ajax': 'true',
        'javax.faces.partial.execute': '@all',
        'javax.faces.partial.render': 'j_idt100',
        'j_idt100:j_idt117': 'j_idt100:j_idt117',
        'j_idt100': 'j_idt100',
        'j_idt100:j_idt102': lastname,
        'j_idt100:j_idt104': surname,
        'j_idt100:j_idt106': email,
        'j_idt100:j_idt108': username,
        "j_idt100:j_idt110": student_id,

        'javax.faces.ViewState': view_state,
        'javax.faces.ClientWindow': jfwid
    }

    headers = {
        'Referer': 'https://moseskonto.tu-berlin.de/moses/administration/personen/suche.html',
        'Faces-Request': 'partial/ajax',
        'Accept': 'application/xml, text/xml, */*; q=0.01',
        'Accept-Language': 'de-DE,en-US;q=0.7,en;q=0.3',
        'X-Requested-With': 'XMLHttpRequest',
        'User-Agent': 'Mozilla/5.0 (X11; Linux x86_64; rv:78.0) Gecko/20100101 Firefox/78.0',
        'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8'
        }

    params = {
        'jfwid': jfwid
    }

    r: requests.Response = session.post(MOSES_SEARCH_URL, data=payload, headers=headers, params=params)
    soup = bs4.BeautifulSoup(r.text, 'lxml')
    soup = soup.find('tbody', attrs={'class': 'ui-datatable-data ui-widget-content'})

    if soup is None:
        return []

    results = []
    for record in soup.findAll('tr'):
        cols = map(lambda c: c.text, record.findAll('td'))
        cols = list(cols)[:-1]  # Drop unnecessary last column
        p = Person(*cols)
        results.append(p)

    return results


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('-u', '--user')
    parser.add_argument('-p', '--password')
    parser.add_argument('emails')
    parser.add_argument('outfile')

    args = parser.parse_args()

    if not args.user:
        args.user = input('Username:')

    if not args.password:
        args.password = getpass.getpass('Password:')

    user, pw, infile, outfile = (args.user, args.password, args.emails, args.outfile)

    session = login(user, pw)

    if session is None:
        print('Login failed :(')
        sys.exit(1)

    print('Login successful!')

    with open(infile, 'r') as fi:
        with open(outfile, 'w') as fo:
            for email in map(lambda l: l.strip(), fi.readlines()):
                ppl = search_people(session, email=email)

                if ppl:
                    fo.write(f'{ppl[0].id}, {email}\n')
                else:
                    fo.write(f'  N/A , {email}\n')
                    print(f'Could not find "{email}"!')


if __name__ == '__main__':
    main()





#!/usr/bin/python
import getpass
import os
import argparse

from .login import login
from .extract import extract_all_submissions

import bs4

ISIS_URL = 'https://isis.tu-berlin.de'
ISIS_SHIB_URL = f'{ISIS_URL}/auth/shibboleth/index.php'
ISIS_ASSIGNMENTS_URL = f'{ISIS_URL}/mod/assign'


def get_assignments(session, course_id):
    url = f"{ISIS_ASSIGNMENTS_URL}/index.php"
    r = session.get(url, params={'id': course_id})
    soup = bs4.BeautifulSoup(r.text, 'html.parser')

    rows = soup.find_all('tr', attrs={'class': ['', 'lastrow']})
    assignments = []
    for r in rows:
        for cell in r.find_all('td', attrs={'class': 'cell c1'}):
            a = {'name': cell.text,
                 'link': cell.a.get('href'),
                 'id': cell.a.get('href').split('id=')[1]}
            assignments.append(a)

    return assignments


def download_submissions(session, assign_id):
    url = f'{ISIS_ASSIGNMENTS_URL}/view.php'
    r = session.get(url, params={'id': assign_id,
                                 'action': 'downloadall'})

    return r.content


def main():
    # NEVES = 13941

    description = '''This script allows you to download any assignment submissions you want
    from a TU-Berlin ISIS course and extracts it into subfolders based on 
    the students group or individually. If the submission is a tar.gz or a zip
    archive it will automatically try to extract it.

    WARNING: The auto-extraction happens without input sanitation. Only use
    this tool if you trust your students.   
    '''
    parser = argparse.ArgumentParser(description=description)

    parser.add_argument('course_id', help='ID of the ISIS course', type=int)
    parser.add_argument('-a', '--all', help='Download submissions for all assignments', action='store_true')
    parser.add_argument('-s', '--separate',
                        help='Extract submission for each student separately instead of one per group.',
                        action='store_true')

    args = parser.parse_args()

    user = input('tubIT-Name: ')
    pw = getpass.getpass(prompt='tubIT-Passwort: ')
    s = login(user, pw, ISIS_URL, ISIS_SHIB_URL)

    if s is None:
        print('Login failed!')
        return
    print('Login Successful')

    assignments = get_assignments(s, args.course_id)
    print('Found {0} assignments.'.format(len(assignments)))

    if not args.all:
        for i, a in enumerate(assignments):
            print(f'[{i}] "{a["name"]}"')
        try:
            selection = int(input('Please select the assignment:'))
        except ValueError:
            print('Invalid selection')
            return -1

        if selection not in range(0, len(assignments)):
            print('Invalid selection')
            return -1

        assignments = [assignments[selection]]

    for a in assignments:
        print(f'Fetching assignment "{a["name"]}"...')
        sub = download_submissions(s, a['id'])
        print('Done.')

        path = f'assignments/{a["name"]}/submissions'
        os.makedirs(path, exist_ok=True)
        extract_all_submissions(path, sub, by_group=(not args.separate))


if __name__ == '__main__':
    main()



import requests


def login(user, pw, url, shib_url):
    s = requests.Session()
    s.get(url)           # Create a flow for this session

    r = s.get(shib_url)  # We will be redirected so save response

    # I have no idea what this does but without this first post we cant login
    payload = {
        'shib_idp_ls_exception.shib_idp_session_ss':'',
        'shib_idp_ls_success.shib_idp_session_ss': True,
        'shib_idp_ls_value.shib_idp_session_ss': '',
        'shib_idp_ls_exception.shib_idp_persistent_ss': '',
        'shib_idp_ls_success.shib_idp_persistent_ss': True,
        'shib_idp_ls_value.shib_idp_persistent_ss': '',
        'shib_idp_ls_supported': '',
        '_eventId_proceed': True
    }
    r = s.post(r.url, data=payload)

    payload = {
        'j_username': user,
        'j_password': pw,
        '_eventId_proceed': ''
    }

    r = s.post(r.url, data=payload)

    for h in r.history:
        for c in h.cookies.keys():
            if '_shibsession_' in c:
                return s

    return None



from setuptools import setup, find_packages

setup(name='tubtools',
      description='Scraping tools for the TU Berlin web portals',
      url='https://gitlab.tubit.tu-berlin.de/bongo0594/tubtools',
      author='Sebastian Bräuer',
      author_email='braeuer@tu-berlin.de',
      license='MIT',
      zip_safe=False,
      install_requires=[
            'fleep',
            'requests',
            'beautifulsoup4'
      ],
      python_requires='>=3.6',
      packages=find_packages(),
      classifiers=[
            "Programming Language :: Python :: 3",
            'Development Status :: 3 - Alpha',
            "License :: OSI Approved :: MIT License",
      ],
      entry_points={
            'console_scripts': [
                  'isisdl = tubtools.isis:main',
                  'moses-export = tubtools.moses:main'
            ]
      })
